/* 
 * Copyright (C) 1997, 1998 Defence Research Establishment, Dept. of Comm.
 * Systems, FOA 72, Sweden.
 * Currently maintained by Christian Jnsson FOA 72 <chj@lin.foa.se>.
 *
 * This file is part of ETFTP version 1.1.3.
 *
 *   ETFTP is free software; you can redistribute it and/or modify it
 *   under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   ETFTP is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with ETFTP; see the file GPL.  If not, write to the
 *   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *   Boston, MA 02111-1307, USA.
 *
 * Code:
 *
 *----------------------------------------------------------------------
 * Filenamne:   cli.c
 * Class:       CLI
 * Created:     0.1, 970503, Per-Erik Forssn
 * Revised:     1.1, 980219, Torbjrn Frostemark
 * 
 * Description: Command Line Interface for client side of ETFTP protocol.
 * Remarks:     See RFC 1986 for details.
 *
 * Private Metods:     -
 * Private Attributes: -
 *----------------------------------------------------------------------
 */

#include "cli.h"
#include "cmdsubr.h"
#include "init.h"
#include "sendrecv.h"
#include "etftptime.h"

void main(argc, argv)
     int   argc;
     char  **argv;
{
  register int    val;
  register int argsl;
  register char	*s;
  register FILE	*fp;

  pName = malloc(80);        /* Reserve memory for pName (name of program) */
  pName = argv[0];
  hostName = malloc(80);
  
  argsl = argc-1;
  argc = 1;

  fp = stdin;
  interactive = 1;

  ClientInitStart();
  
  CLIDISP("ETFTP at your command.\n");
  while (argsl > 0 && (*argv[argc] == '-')) {
    argsl--;			/* used one token */
    argc++;
    for (s = argv[argc-1]+1; *s != '\0'; s++) {
      switch (*s) {
      case 'h':		        /* specify host name */
	if (argsl < 0) {
	  printf("-h requires another argument");
	  exit(0);
	}
	strcpy(hostName,argv[argc]);
	argsl--;		/* used one token */
	argc++;
	break;
      case 't':
	traceFlag = TRUE;
	break;
      case 'v':
	verboseFlag = TRUE;
	break;
      case 'H':
	hashFlag = TRUE;
	break;
      case 'f':
	if (argsl <= 0) {
	  printf("-f requires another argument");
	  exit(0);
	}
	if ( (fp = fopen(argv[argc],"r")) == NULL ) {
	  printf("%s: can't open %s for reading\n", pName,argv[argc]);	  
	  fp = stdin;
	  interactive=1;
	} else {
	  printf("%s: batch file set to : %s\n",pName,argv[argc]);
	  interactive=0;
	}
	argsl--;		/* used one token */
	argc++;
	break;
      default:
	printf("unknown command line option: %c\n", *s);
	break;
      }
    }
  }

  /* connect if args */
  if (argsl > 0 ) {
    strcpy(hostName,argv[argc]);
    argsl--;			/* used one token */
    argc++;
    if(gethostbyname(hostName) == NULL) {
      printf("hostname not recognized.\n");
      exit(0);
    }
    /* no we can look on ourselves as connected */
    connected=1;
    if (argsl > 0) {		/* port number supplied? */
      val = atoi(argv[argc]);
      if (val <= 0) {
	printf("invalid port number %s.\n",argv[argc]);
	val = 1818;
      }
      remotePort = val;
      printf("Connected to %s on port %d.\n", hostName, remotePort);
      argsl--;			/* used one token */
      argc++;
    } else {			/* only hostname */
      printf("Connected to %s.\n", hostName);
    }
  }

  /* run it */
  mainLoop(fp); 
}

/*
 *----------------------------------------------------------------------
 * mainLoop
 *
 * Reads commandlines from fp and executes them.
 * 
 * Results:
 *      None. 
 *
 * Side effects:
 *      None.
 *----------------------------------------------------------------------
 */

void mainLoop(fp)
     FILE *fp;
{
  
  /*
   * Main loop.  Read a command and execute it.
   * This loop is terminated by a "quit" command, or an
   * end-of-file on the command stream.
   */
  
  if (setjmp(jmpMainLoop) < 0) {
    printf("Timeout\n");
  }
  
  CLIDISP1("%s", prompt);
  
  /* command is initialized in init.c as 80 chars */
  
  while (CSGetLine(fp)) {
    if (CSGetToken(command) != NULL) {
      /*if (signal(SIGINT, SIG_IGN) != SIG_IGN) {
	signal(SIGINT,(sighandler_t) sig_intr);
	}*/
      signal(SIGINT,(sighandler_t) sig_intr);
      CSDoCmd(command);
      signal(SIGALRM, SIG_IGN);	/* first ignore the signal */
      alarm(0);			/* then assure alarm is off */
    }
    
    CLIDISP1("%s", prompt);
  }
}

/*
 *----------------------------------------------------------------------
 * sig_intr
 *
 * 	INTR signal handler.  Just return to the main loop above.
 * 	In case we were waiting for a read to complete, turn off any possible
 * 	alarm clock interrupts.
 *
 * 	Note that with TFTP, if the client aborts a file transfer (such as with
 * 	the interrupt signal), the server is not notified.  The protocol counts
 * 	on the server eventually timing out and exiting.
 *	IN ETFTP QUIT OR ABORT SHOULD BE SENT!!!!
 *
 * Results:
 *      None.
 *
 * Side effects:
 *      None.
 *----------------------------------------------------------------------
 */

int sig_intr()
{
  signal(SIGALRM, SIG_IGN);	/* first ignore the signal */
  alarm(0);			/* then assure alarm is off */

  strcpy(errString, "Ctrl-C pressed.");
  SendABORT();

  printf("\nCtrl-C pressed -> ABORT sent!!\n");

  if (!interactive) {
    /* show elapsed time and number of bytes received so far */
    ctrlTime = TimeGet() - ctrlTime;
    printf("Elapsed time = %d s, %ld bytes received (%ld bytes left).\n", ctrlTime,
	   bytesReceived, totNBytes - bytesReceived);
  }

  printf("File transfer aborted.\n");
  fputc('\n',stdout);
  longjmp(jmpMainLoop, 1);
  /* NOTREACHED */
}
