/* 
 * Copyright (C) 1997, 1998 Defence Research Establishment, Dept. of Comm.
 * Systems, FOA 72, Sweden.
 * Currently maintained by Christian Jnsson FOA 72 <chj@lin.foa.se>.
 *
 * This file is part of ETFTP version 1.1.3.
 *
 *   ETFTP is free software; you can redistribute it and/or modify it
 *   under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   ETFTP is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with ETFTP; see the file GPL.  If not, write to the
 *   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *   Boston, MA 02111-1307, USA.
 *
 * Code:
 *
 *----------------------------------------------------------------------
 * Filenamne:   clu.c
 * Module:      CLU (Common Log Unit)
 * Created:     0.1, 970418, Torbjrn Frostemark
 * Revised:     1.1, 980219, Torbjrn Frostemark
 * 
 * Description: Log unit used to log events.
 * Remarks:     -
 *
 * Private Methods:	-
 *
 * Private Attributes:	FILE *logFD
 *----------------------------------------------------------------------
 */

#include "clu.h"
#include "file.h"
#include "etftptime.h"

FILE *logFD;			/* UNIX file descriptor for log file */
FILE *debugFD;			/* UNIX file descriptor for debug file */

/*
 *----------------------------------------------------------------------
 * CLUStart(char *fileName)
 *
 *      Create new log file on file system.
 *
 * Results:
 *	New log file and debug file (server case).
 *
 * Side effects:
 *	Creates a new file in file system.
 *----------------------------------------------------------------------
 */

void CLUStart()
{
  char fName[80];		/* log file name */
  char service[20];		/* service to be logged */
  char data[80];		/* data to be logged */
  char logFormat[80];		/* describes format in log file */

#ifdef CLIENT
  strcpy(service, "client");
  strcpy(logFormat, "Format: avgRate momRate avgDel momDel aveLoss momLoss\n\n");

  sprintf(fName, "log.%s.%ld", service, TimeGet()); /* create log file name */

  if ((logFD = FileCreate(fName)) == NULL) {
    traceFlag = FALSE;	/* create new log file */
    printf("The common log unit could not create log file, trace now off.\n");
    return;
  }

  sprintf(data, "Captain's log, star date: %s\n\n", TimeDate()); /* header */
  FileWrite(logFD, data, strlen(data), MODE_BINARY);

  strcpy(data, logFormat);	/* log format */
  FileWrite(logFD, data, strlen(data), MODE_BINARY);

  fflush(logFD);
#endif /* CLIENT */

#ifdef SERVER
  strcpy(service, "server");
  strcpy(logFormat, "Format: avgRate momRate avgDel momDel aveLoss momLoss\n\n");

  if(verboseFlag) {
    /* Create debug file */
    sprintf(fName, "/var/log/debug.etftpd.%ld", TimeGet()); /* create log file name */
    if ((debugFD = FileCreate(fName)) == NULL) {
      verboseFlag = FALSE;
      printf("The common log unit could not create debug file, verbose now off.\n");
    }
  }

  if (traceFlag) {
    sprintf(fName, "log.%s.%ld", service, TimeGet()); /* create log file name */

    if ((logFD = FileCreate(fName)) == NULL) {
      traceFlag = FALSE;	/* create new log file */
      printf("The common log unit could not create log file, trace now off.\n");
      return;
    }
    
    sprintf(data, "Captain's log, star date: %s\n\n", TimeDate()); /* header */
    FileWrite(logFD, data, strlen(data), MODE_BINARY);
    
    strcpy(data, logFormat);	/* log format */
    FileWrite(logFD, data, strlen(data), MODE_BINARY);
    
    fflush(logFD);
  }
#endif /* SERVER */

}

/*
 *----------------------------------------------------------------------
 * CLUStop
 *
 *	Stop logging.
 *
 * Results:
 *	Close log file.
 *
 * Side effects:
 *	Closes file on file system.
 *----------------------------------------------------------------------
 */

void CLUStop()
{
  char data[80];		/* data to be put in log */

  sprintf(data, "\nLog closed: %s\n", TimeDate());
  FileWrite(logFD, data, strlen(data), MODE_BINARY);

  fflush(logFD);

  FileClose(logFD);		/* close log */

  if (verboseFlag) {
    FileClose(debugFD);
  }
}

/*
 *----------------------------------------------------------------------
 * CLUPut
 *
 *	Put data into log.
 *
 * Results:
 *	Data is written in log file.
 *
 * Side effects:
 *	File on file system is changed.
 *----------------------------------------------------------------------
 */

void CLUPut(stats)
     RecvStat *stats;		/* log struct */
{
  char data[80];		/* log data */

  sprintf(data, "%s: %5.2f  n/a  n/a  n/a  n/a  n/a\n", TimeDate(),
	  stats->aveRate);
  FileWrite(logFD, data, strlen(data), MODE_BINARY); /* write log */

  fflush(logFD);
}

/*
 *----------------------------------------------------------------------
 * CLUPutStr
 *
 *	Put string into log.
 *
 * Results:
 *	String is written in log file.
 *
 * Side effects:
 *	File on file system is changed.
 *----------------------------------------------------------------------ds
 */

void CLUPutStr(s)
     char *s;
{
  char data[80];

  sprintf(data, "%s\n", s);

  FileWrite(logFD, data, strlen(data), MODE_BINARY); /* write log */
  fflush(logFD);
}






