/* 
 * Copyright (C) 1997, 1998 Defence Research Establishment, Dept. of Comm.
 * Systems, FOA 72, Sweden.
 * Currently maintained by Christian Jnsson FOA 72 <chj@lin.foa.se>.
 *
 * This file is part of ETFTP version 1.1.3.
 *
 *   ETFTP is free software; you can redistribute it and/or modify it
 *   under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   ETFTP is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with ETFTP; see the file GPL.  If not, write to the
 *   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *   Boston, MA 02111-1307, USA.
 *
 * Code:
 *
 *----------------------------------------------------------------------
 * Filenamne:	getput.c
 * Class:	ClientGetPut
 * Created:	0.1, 970418, Torbjrn Frostemark
 * Revised:	1.1, 980219, Torbjrn Frostemark
 * 
 * Description:	File get/put processing.
 *
 * 		This is the way the client side gets started - either
 *		the user wants to get a file (generates a RRQ command
 *		to the server) or the user wants to put a file (generates
 *		a WRQ command to the server).  Once either the RRQ or the
 *		WRQ command is sent, the finite state machine takes over
 *		the transmission.
 *
 * Remarks:	-
 *
 * Private methods: 	-
 *
 * Private attributes:	-
 *----------------------------------------------------------------------
 */

#include        "clu.h"
#include	"getput.h"
#include	"etftptime.h"
#include	"file.h"
#include	"fsm.h"
#include	"sendrecv.h"
#include	"net.h"

/*
 *----------------------------------------------------------------------
 * DoGet
 *
 *      Execute a get command - read a remote file and store on the
 *	local system.
 *
 * Results:
 *	If successful transfer, file is stored on file system.
 *
 * Side effects:
 *	None.
 *----------------------------------------------------------------------
 */

void DoGet(remFileName,locFileName)
     char *remFileName;
     char *locFileName;
{
  long getStartTime, getTimeDiff;

  sesRemotePort=remotePort;	/* Port number for this session */
  if ((localFP = FileCreate(locFileName)) == NULL) {
    /* cannot create file */
    printf("etftp: Cannot create file %s.\n", locFileName);
    DEBUG0("Error exiting DoGet.");
    return;
  }
  
  if ((outSD = ClientNetOpen(hostName, ETFTP_SERVICE, sesRemotePort)) < 0) {
    printf("etftp: Failed to open port.\n");
    DEBUG0("Error exiting DoGet.");
    return;
  }

  getStartTime = TimeGet();	/* start timer for statistics */
  
  ctrlTime = getStartTime;	/* showed when Ctrl-C is pressed */

  if (SendOPEN(remFileName, totNBytes, MODE_GET) == EXIT_SUCCESS) {
    /* server wants to connect, start FSM in OPEN state */
    if (FSMLoop(PT_OPEN) == EXIT_SUCCESS) {
      getTimeDiff = TimeGet() - getStartTime; /* stop timer for statistics */
      printf("etftp: Received %ld bytes in %ld seconds (%ld bytes/s).\n", totNBytes, getTimeDiff,
	     totNBytes/getTimeDiff);
    } else {
      printf("\nComplete transfer failed. File may be corrupted.\n");
    }
  }
  NetClose();
  FileClose(localFP);
}

/*
 *----------------------------------------------------------------------
 * DoPut
 *
 *	Execute a put command - send a local file to the remote system.
 *
 * Results:
 *	If successful transfer, file is sent to reciever.
 *
 * Side effects:
 *	None.
 *----------------------------------------------------------------------
 */

void DoPut(remFileName,locFileName)
     char *remFileName;
     char *locFileName;
{
  long getStartTime, getTimeDiff;
  DEBUG1("Putting on port %d\n",remotePort);
  sesRemotePort = remotePort;	/* Port number for this session */

  totNBytes = FileCount(locFileName,modeType);	/* set file size */

  if ((localFP = FileOpen(locFileName)) == NULL) {
    /* cannot open file */
    printf("etftp: Cannot open file %s.\n", locFileName);
    DEBUG0("Error exiting DoPut.");
    return;
  }
  
  if ((outSD = ClientNetOpen(hostName, ETFTP_SERVICE, sesRemotePort)) < 0) {
    printf("etftp: Failed to open port.\n");
    DEBUG0("Error exiting DoPut.");
    return;
  }
  
  getStartTime = TimeGet();	/* start timer for statistics */

  ctrlTime = getStartTime;	/* showed when Ctrl-C is pressed */

  lastSend = MAXDATA;
  if (SendOPEN(remFileName, totNBytes, MODE_PUT) == EXIT_SUCCESS) {
    /* server wants to connect, start FSM in RESPONSE state */
    if (FSMLoop(PT_OPEN) == EXIT_SUCCESS) {
      getTimeDiff = TimeGet() - getStartTime; /* stop timer for statistics */  
      printf("etftp: Sent %ld bytes in %ld seconds (%ld bytes/s).\n", totNBytes, getTimeDiff,
	     totNBytes/getTimeDiff);
    } else {
      printf("\nComplete transfer failed. File may be corrupted.\n");
    }
  }
  NetClose();
  FileClose(localFP);
}





