/* 
 * Copyright (C) 1997, 1998 Defence Research Establishment, Dept. of Comm.
 * Systems, FOA 72, Sweden.
 * Currently maintained by Christian Jnsson FOA 72 <chj@lin.foa.se>.
 *
 * This file is part of ETFTP version 1.1.3.
 *
 *   ETFTP is free software; you can redistribute it and/or modify it
 *   under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   ETFTP is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with ETFTP; see the file GPL.  If not, write to the
 *   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *   Boston, MA 02111-1307, USA.
 *
 * Code:
 *
 *----------------------------------------------------------------------
 * Filenamne:   init.c
 * Module:      ServerInit/ClientInit
 * Created:     0.1, 970418, Torbjrn Frostemark
 * Revised:     1.1, 980219, Torbjrn Frostemark
 * 
 * Description: Initialize objects.
 * Remarks:     -
 *
 * Private methods:	-
 *
 * Private attributes:	-
 *----------------------------------------------------------------------
 */

#include "init.h"

/*
 * Global attributes.
 */

#ifdef SERVER
jmp_buf	jmpMainLoop;		/* to return to main command loop */
int	lastSend;		/* #bytes of data in last data packet */
FILE	*localFP;		/* fp of local file to read or write */
int	nextBlkNum;		/* next block# to send/receive */
int	pkSent;			/* last opcode sent */
int	pkRecv;			/* last opcode received */
char	recvBuff[];		/* list of recieving buffers */
char	sendBuff[];		/* list of sending buffers */

long	totNBytes;		/* for get/put statistics printing */
int	port;		        /* local port number */
long	connectID;		/* session ID */
int     inetdFlag;              /* started by inetdemon */
int	multiBuf;		/* number of multi buffers */
unsigned short	bufferSize;	/* no. of blocks per buffer */
unsigned short	burstSize;	/* Current burst size */
long 	baudRate;		/* transfer speed */
unsigned short	blockSize;
unsigned short	burstRate;
short	localPort;		/* local port number */
short	remotePort;		/* remote port number */
short	sendLen;		/* number of bytes in sendPack[] */

/* FLAGS */
int	timeoutFlag;		/* net timeout */
int	traceFlag;		/* -t command line option, or "trace" cmd */
int	modeType;		/* see MODE_xxx values */
unsigned short	radioDelay;	/* estimated delay in seconds */
int	verboseFlag;		/* -v command line option */
int     connected;
short	renegotiate;		/* toggle adaptivity */
char	*recvPack;		/* temporary storage of recieved packet */
char	*sendPack;		/* temporary storage of packet to be sent */
char	*hostName;		/* name of host system */
int	outSD;			/* Out socket */
unsigned short packetSCounter;  /* counts number of packets sent */
unsigned short packetRCounter;  /* counts number of packets received */

/* Stuff needed by sendrecv */
long	bufferNum;              /* number of buffer (running) */
short	sequenceNum;            /* number of controll messages (running) */
short	tightTimer;             /* See RFC for details */
short	looseTimer;             /* See RFC for details */
long	rtTimer;		/* See RFC for details */
int	transferMode;		/* get/put. Used to measure rtt */
int	rttSet;			/* Used to measure rtt */ 
#endif /* SERVER */

#ifdef CLIENT
char    *command;		/* the command being processed */
int     connected;		/* true if we're connected to host */
char    *hostName;		/* name of host system */
int     inetdFlag;		/* true if we were started by a daemon */
int     interactive;		/* true if we're running interactive */
jmp_buf jmpMainLoop;		/* to return to main command loop */
int     modeType;		/* see MODE_xxx values */
char    *pName;			/* the name by which we are invoked */
char    *prompt;		/* prompt string, for interactive use */
int     verboseFlag;		/* -v command line option */
int	traceFlag;		/* -t command line option, or "trace" cmd */
int	hashFlag;		/* -h command line option, or "hash" cmd */
short   renegotiate;            /* toggle activity */
unsigned short	blockSize;	/* Current data-block size */
unsigned short	bufferSize;	/* No of blocks per buffer */
unsigned short	burstSize;	/* Current burst size */
int	multiBuff;		/* No of buffers */
long	baudRate;		/* transfer speed */
unsigned short	radioDelay;	/* Estimated turnover time */
/* Surplus */
int	lastSend;		/* #bytes of data in last data packet */
FILE	*localFP;		/* fp of local file to read or write */
int	nextBlkNum;		/* next block# to send/receive */
int	pkSent;			/* last opcode sent */
int	pkRecv;			/* last opcode received */
char	recvBuff[];		/* list of recieving buffers */
char	sendBuff[];		/* list of sending buffers */
short	sendLen;		/* number of bytes in sendPack[] */
char	tempToken[80];		/* present token */
long	totNBytes;		/* for get/put statistics printing */
short	localPort;		/* local port number */
short	remotePort;		/* remote port number */
short	sesRemotePort;          /* Remote port for current session */
long	connectID;		/* session ID */
unsigned short	burstRate;
char	*recvPack;		/* temporary storage of recieved packet */
char	*sendPack;		/* temporary storage of packet to be sent */
unsigned short packetSCounter;  /* counts number of packets sent */
unsigned short packetRCounter;  /* counts number of packets received */

/* Stuff needed by sendrecv */
long  bufferNum;                /* number of buffer (running) */
short sequenceNum;              /* number of controll messages (running) */
short tightTimer;               /* See RFC for details */
short looseTimer;               /* See RFC for details */
long  rtTimer;			/* See RFC for details */
int	outSD;			/* Out socket */
int	ctrlTime;		/* when Ctrl-C is pressed */
long	bytesReceived;		/* bytes received counter */
#endif /* CLIENT */

#ifdef	SERVER
/*
 *----------------------------------------------------------------------
 *
 * ServerInitStart
 *
 *      Initialize server.
 *
 * Results:
 *	Global variables are set for server.
 *      
 * Side effects:
 *	Many global variables are set.
 *
 *----------------------------------------------------------------------
 */

int ServerInitStart()
{

  /* flags etc */
  renegotiate=1;		/* On */
  modeType=MODE_BINARY;         /* MODE_BINARY */
  timeoutFlag = 1;		/* net timeout */
  traceFlag = FALSE;
  verboseFlag = FALSE;

  /* initate etc */
  inetdFlag=1;                  /* Default started by inry*/
  connected=0;			/* Not connected yet */
  hostName=malloc(128);
  /*  recvPack=calloc(MAXPACKETSZ,1); */
  /*  sendPack=calloc(MAXPACKETSZ,1); */
  recvPack=calloc(MAXBUFF,1);
  sendPack=calloc(MAXBUFF,1);
  remotePort=1818;		/* Default port */
  localPort=1818;

  /* "standard" settings */
  blockSize=512;
  bufferSize=128;
  burstSize=8;
  multiBuf=1;
  baudRate=16000;
  radioDelay=2;

  /* reset just for safety */
  connectID=0;
  lastSend=0;
  totNBytes=0;
  pkSent=0;
  pkRecv=0;
  nextBlkNum=0;

 /* stuff for sendrecv */
  bufferNum=0;
  sequenceNum=0;
  rttSet = TRUE;

  /* XXX those should be changed */
  looseTimer=0;
  tightTimer=0;
  outSD=-1;
  return EXIT_SUCCESS;
}
#endif /* SERVER */

#ifdef CLIENT
/*
 *----------------------------------------------------------------------
 * ClientInitStart
 *
 *      Initialize client.
 *
 * Results:
 *	Global variables are set for client.
 *
 * Side effects:
 *	Many global variables are set.
 *----------------------------------------------------------------------
 */

int ClientInitStart()
{
  connected=0;			/* Not connected yet */
  hostName=malloc(128);
  inetdFlag=0;			/* Client is never started by demon */
  remotePort=1818;		/* Default port */
  localPort=1818;
  modeType=MODE_BINARY;         /* MODE_BINARY */
  renegotiate=1;		/* On */
  blockSize=512;		
  bufferSize=128;
  burstSize=8;
  multiBuff=2;
  baudRate=16000;
  radioDelay=2;
  prompt=malloc(50);
  strcpy(prompt,"etftp> ");
  command=malloc(80);           /* Reserve memory for command */
  /*  recvPack=calloc(MAXPACKETSZ,1); */
  /*  sendPack=calloc(MAXPACKETSZ,1); */
  recvPack=calloc(MAXBUFF,1);
  sendPack=calloc(MAXBUFF,1);
  traceFlag = FALSE;
  verboseFlag = FALSE;
  hashFlag = FALSE;
  packetSCounter = 0;
  packetRCounter = 0;
  bytesReceived = 0;

  /* stuff for sendrecv */
  bufferNum=0;
  sequenceNum=0;

  /* XXX those should be changed */
  looseTimer=0;
  tightTimer=0;
  outSD=-1;
  return EXIT_SUCCESS;
}
#endif /* CLIENT */
