/* 
 * Copyright (C) 1997, 1998 Defence Research Establishment, Dept. of Comm.
 * Systems, FOA 72, Sweden.
 * Currently maintained by Christian Jnsson FOA 72 <chj@lin.foa.se>.
 *
 * This file is part of ETFTP version 1.1.3.
 *
 *   ETFTP is free software; you can redistribute it and/or modify it
 *   under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   ETFTP is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with ETFTP; see the file GPL.  If not, write to the
 *   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *   Boston, MA 02111-1307, USA.
 *
 * Code:
 *
 *----------------------------------------------------------------------
 * Filenamne:   servmain.c
 * Class:       ServerMain
 * Created:     0.1, 970504, Einar Brandt 
 * Revised:     1.1, 980219, Torbjrn Frostemark
 * 
 * Description: Server side. Invokes serverFSM.
 * 
 * Command parameters are:
 *	-i		says we were *not* started by inted.
 *	-l port#	specifies local port# to listen on.
 *      -r port#        specifies remote port# to listen on.
 *	-t		turns on the traceFlag - writes to a file.
 *      -v              turns on the verboseFlag - debug info.
 *	-b		specifies block size
 *
 * Private Metods:     None.
 * Private Attributes: None.
 *----------------------------------------------------------------------
 */

#include "servmain.h"
#include "init.h"
#include "net.h"
#include "clu.h"
#include "fsm.h"

/*
 *------------------------------------------------------------------
 * main
 *
 *	Main function.
 *
 * Results:
 *	Start server.
 *
 * Side effects:
 *	None.
 *------------------------------------------------------------------
 */

void main(argc, argv)
     int   argc;
     char  **argv;
{	
  register char	*s;
  int		childpid;

  /* Initiate server */
  DEBUG0("Initiating server.");
  ServerInitStart();
  
  /* Scan command line */
  while (--argc > 0 && (*++argv)[0] == '-') {
    for (s = argv[0]+1; *s != '\0'; s++) {
      switch (*s) {
      case 'i':
	inetdFlag = 0;		/* turns OFF the flag */
				/* (it defaults to 1) */
	break;
	
      case 'l':			/* specify server's local port number */
	if (--argc <= 0)
	  {
	    DEBUG0("-l requires an argument");
	    exit;
	  }
	localPort = atoi(*++argv);
	break;

      case 'r':			/* specify server's remote port# */
	if (--argc <= 0)
	  {
	    DEBUG0("-r requires an argument");
	    exit;
	  }
	remotePort = atoi(*++argv);
	break;
	
      case 'b':			/* specify server's block size value */
	if (--argc <= 0)
	  {
	    DEBUG0("-b requires an argument");
	    exit;
	  }
	blockSize = atoi(*++argv);
	break;

      case 't':
	traceFlag = TRUE;
	break;

      case 'v':
	verboseFlag = TRUE;
	break;
	
      default:
	DEBUG1("unknown command line option: %c\n", *s);
      }
    }
  }
  /* command line options decoded */

  if ( traceFlag || verboseFlag ) {
    CLUStart();
    }

  if (!inetdFlag) {
    /*
     * Start us up as a daemon process (in the background).
     */
     
    NetInit("ETFTP",localPort);
  }
  
  /* If traceFlag then start logging */
  /* If verboseFlag then start writing debug info */

  /*
   * Concurrent server loop.
   * The child created by NetListen() handles the client's request.
   * The parent waits for another request.  In the inetd case,
   * the parent from NetListen() never returns.
   */
  
  while (TRUE) {
    DEBUG0("Main server: Waiting for requests.");

    if ( (childpid = NetListen(inetdFlag)) == 0) {
      DEBUG0("Got request, starting FSM.");
      FSMLoop(-1);		/* child processes client's request */
      NetClose();		/* then we're done */
      if ( traceFlag ) {
	CLUStop();		/* Close log file */
      }
      DEBUG0("Transfer session ended.");
      exit(0);      
    }
    DEBUG0("Parent out...");
    /* parent waits for another client's request */
  }
  /* NOTREACHED */
}

